# frozen_string_literal: true

# :nocov:
class Renuo::Cli::Commands::CheckDeploioStatus
  TIMEOUT_IN_SECONDS = 600
  INTERVAL_IN_SECONDS = 30

  APP_NAME = ENV.fetch "DEPLOIO_APP_NAME", nil
  PROJECT = ENV.fetch "DEPLOIO_PROJECT", nil

  command "check-deploio-status" do |c|
    c.syntax = "renuo check-deploio-status"
    c.summary = "Checks the build and release status of the deployment."
    c.description = "Checks the build and release status of the deployment."
    c.option "--git-revision <revision>", String, "The git revision to check"
    c.action { |_, options| new(options).run }
  end

  def initialize(options)
    abort "missing env DEPLOIO_APP_NAME" if APP_NAME.nil?
    abort "missing env DEPLOIO_PROJECT" if PROJECT.nil?
    @revision = options.git_revision || `git rev-parse HEAD`.strip
  end

  def run
    puts "(1/2) Checking build status for revision #{@revision}..."
    poll "build"
    abort "build check timed out after #{TIMEOUT_IN_SECONDS} seconds" if build.nil?

    puts "(2/2) Checking release status for build #{build_name}..."
    poll "release"
    abort "release check timed out after #{TIMEOUT_IN_SECONDS} seconds" if release.nil?
  end

  private

  def system!(cmd)
    abort unless system cmd
  end

  def fetch(type)
    command = "nctl get #{type} -a #{APP_NAME} -p #{PROJECT} -o yaml"
    stdout, stderr, status = Open3.capture3 command

    abort "error fetching #{type} information: #{stderr}" unless status.success?
    Psych.load_stream stdout
  end

  def fetch_build_logs
    command = "nctl logs build -a #{APP_NAME} -p #{PROJECT} -l 5000 --no-labels"
    stdout, stderr, status = Open3.capture3 command

    abort "error fetching build logs: #{stderr}" unless status.success?
    stdout
  end

  def build
    @build ||= fetch("builds").find do |build|
      build.dig("spec", "forProvider", "sourceConfig", "git", "revision") == @revision
    end
  end

  def build_name
    @build_name ||= build.dig("metadata", "name")
  end

  def build_status
    build.dig("status", "atProvider", "buildStatus")
  end

  def release
    @release ||= fetch("releases").find do |release|
      release.dig("spec", "forProvider", "build", "name") == build_name
    end
  end

  def release_status
    release.dig("status", "atProvider", "releaseStatus")
  end

  def succeeded?(status, type) # rubocop:disable Metrics/MethodLength
    case status
    when "available", "success"
      puts "#{type} succeeded"
      true
    when "error", "failed", "failure"
      puts fetch_build_logs
      abort "#{type} failed"
    else
      puts "#{type} status is #{status}, waiting..."
      instance_variable_set :"@#{type}", nil
    end
  end

  def poll(type)
    elapsed = 0

    while elapsed < TIMEOUT_IN_SECONDS
      if send type
        break if succeeded?(send(:"#{type}_status"), type)
      else
        puts "no matching #{type} found, waiting..."
      end

      sleep INTERVAL_IN_SECONDS
      elapsed += INTERVAL_IN_SECONDS
    end
  end
end
# :nocov:
