# frozen_string_literal: true

# :nocov:
class Renuo::Cli::Commands::CreateAwsProject
  command "create-aws-project" do |c|
    c.syntax = "renuo create-aws-project"
    c.summary = "Generates necessary commands for our project setup on AWS incl. necessary installations."
    c.description = <<~DESCRIPTION
      This creates commands for creating AWS users, buckets an versioning policies and CloudFront.
      It also guides you to set up the necessary environment.

      You will be asked for:
      (- installation of aws-cli, if not yet done)
      (- setting up aws user `renuo-app-setup`, if not yet done)
      - project name and suffix so that the script can respect our naming conventions
      - the Redmine project name to tag buckets for AWS billing references
      - whether you want to setup CloudFront to deliver assets via S3

      The generated commands do the following:
      - create an IAM user for each environment (main, develop) and add it to the renuo apps group.
      - create S3 buckets for each user who owns it
      - tag the buckets
      - enable versioning for main buckets
      (- set up a CloudFront distribution for each environment with the default config or plus alias if configured)
    DESCRIPTION
    c.example "Setup a project (you will be asked for details)", "renuo create-aws-project"
    c.action { new.run }
  end

  def initialize
    ensure_aws_setup?
    ensure_aws_profile_existing?
    collect_project_information
  end

  def run
    %w[main develop].each do |branch|
      print_setup_commands branch
    end
  end

  private

  def collect_project_information
    say "We need now some informations to create your s3-bucket:".colorize :green
    collect_general_information
    collect_s3_information
    collect_cloudfront_information
  end

  def collect_general_information
    @project_name = ask("Project name (eg: renuo-cli): ") { |q| q.validate = /.+/ }
  end

  def collect_s3_information
    @project_purpose = ask('Suffix describing a non-default purpose (eg: "archives"): ') { |q| q.default = "none" }
    @project_purpose = nil if @project_purpose.empty? || @project_purpose == "none"
    @redmine_project = ask("Redmine project name for billing (eg: internal): ") do |q|
      q.default = @project_name
      q.validate = /.+/
    end
    @aws_profile = "renuo-app-setup"
    @aws_region = ask("AWS bucket region: ") { |q| q.default = "eu-central-1" }
    @aws_app_group = "renuo-apps-v2"
  end

  def collect_cloudfront_information
    @setup_cloudfront = agree("Would you like to setup also AWS CloudFront? " \
                              "(Default: yes, if you want to deliver assets/images from S3)")
  end

  def ensure_aws_setup?
    ensure_is_setup?("aws --version > /dev/null",
                     "brew install awscli",
                     "Would you like to install aws-cli via `brew install awscli`?")
  end

  def ensure_aws_profile_existing?
    ensure_is_setup?("aws configure --profile renuo-app-setup list > /dev/null",
                     "aws configure --profile renuo-app-setup",
                     "Would you like set up a profile for AWS " \
                     "`aws configure --profile renuo-app-setup`? (User/Password in keystore)")
  end

  def ensure_is_setup?(installation_check_command, installation_command, agree_text)
    (system(installation_command) if agree(agree_text)) until system(installation_check_command)
  end

  def print_setup_commands(branch) # rubocop:todo Metrics/AbcSize
    say "\n# AWS #{branch} \n".colorize :green
    # wrap_at inserts newlines, after that, the command is no longer copyable
    $terminal.wrap_at = nil if $terminal
    puts aws_iam_setup(@aws_profile, aws_user(branch), @aws_app_group)
    puts aws_s3_setup(@aws_profile, aws_user(branch), @aws_region, @redmine_project)
    say "# Hint: Don't forget to copy SecretAccessKey and AccessKeyId".colorize :yellow
    puts aws_s3_versioning_setup(@aws_profile, aws_user(branch)) if branch == "main"
    cloudfront_setup(aws_user(branch))
    $terminal.wrap_at = :auto if $terminal
  end

  def cloudfront_setup(bucket)
    return unless @setup_cloudfront

    puts aws_cloudfront_setup(@aws_profile, bucket, @redmine_project)
    say "# Hint: Find the distribution domain name under Distribution.DomainName".colorize :yellow
  end

  def aws_user(branch)
    [@project_name, branch, @project_purpose].compact.join("-")
  end

  def aws_iam_setup(profile, user, app_group)
    <<~IAM_COMMANDS
      aws --profile #{profile} iam create-user --user-name #{user}
      aws --profile #{profile} iam add-user-to-group --user-name #{user} --group-name #{app_group}
      aws --profile #{profile} iam create-access-key --user-name #{user}
    IAM_COMMANDS
  end

  def aws_s3_setup(profile, bucket, region, redmine_project)
    <<~S3_COMMANDS
      aws --profile #{profile} s3 mb s3://#{bucket} --region #{region}
      aws --profile #{profile} s3api put-bucket-tagging --bucket #{bucket} --tagging "TagSet=[{Key=redmine_project,Value=#{redmine_project}}]"
    S3_COMMANDS
  end

  def aws_s3_versioning_setup(profile, bucket)
    <<~S3_VERSIONING_COMMANDS
      aws --profile #{profile} s3api put-bucket-versioning --bucket #{bucket} --versioning-configuration Status=Enabled
    S3_VERSIONING_COMMANDS
  end

  def aws_cloudfront_setup(profile, bucket, redmine_project)
    tags = [{ Key: "redmine_project", Value: redmine_project }]
    cloudfront_config_string = Renuo::Cli::Services::CloudfrontConfigService.new(bucket, tags).to_s

    <<~CLOUDFRONT_COMMANDS
      aws --profile #{profile} cloudfront create-distribution-with-tags --distribution-config-with-tags '#{cloudfront_config_string}'
    CLOUDFRONT_COMMANDS
  end
end
# :nocov:
