# frozen_string_literal: true

class Renuo::Cli::Commands::CreatePr
  command "create-pr" do |c|
    c.syntax = "renuo create-pr"
    c.summary = "Creates a PR for the current branch"
    c.description = "Creates a PR, assigns it to you and automatically inserts the Redmine ticket number into " \
                    "the PR body"
    c.option "--title <title>", String, "The title of the pull request (optional)"
    c.option "--redmine-ticket <number>", Integer, "The redmine ticket number (optional)"
    c.option "--draft", "Mark the PR as draft"
    c.option "--[no-]web-create", "Open web browser to create a pull request (default: true)"
    c.example "Create a PR with the title 'Implement XYZ'",
              "renuo create-pr --title 'Implement XYZ'"
    c.action do |_, options|
      options.default web_create: true
      new.run(options)
    end
  end

  def run(opts)
    create_pr(opts)
    open_pr_in_browser unless opts.web_create
  end

  private

  def create_pr(opts)
    command = [
      "gh pr create",
      "--assignee @me",
      pr_title(opts),
      "--body \"#{pr_body(opts)}\"",
      ("--web" if opts.web_create),
      ("--draft" if opts.draft)
    ].compact.join(" ")

    puts `#{command}`
  end

  def pr_title(opts)
    opts.title ? "--title \"#{opts.title}\"" : "--fill"
  end

  def pr_body(opts)
    redmine_ticket_number = redmine_ticket_number(opts)
    return "" unless redmine_ticket_number

    "TICKET-#{redmine_ticket_number}"
  end

  def redmine_ticket_number(opts)
    current_branch = `git branch --show-current`.strip
    opts.redmine_ticket || current_branch.match(/(\d+)/)&.captures&.first
  end

  def open_pr_in_browser
    puts `gh pr view --web`
  end
end
