# frozen_string_literal: true

class Renuo::Cli::Commands::GithubReplace
  command "github-replace" do |c|
    c.syntax = "renuo github-replace"
    c.summary = "Replaces text in multiple GitHub projects and automatically creates a pull request"
    c.description = "Finds and replaces the contents of specific files in Renuo GitHub projects and " \
                    "automatically creates a pull request"
    c.option "--projects <name,name,..>", String, "The github project to clone and edit"
    c.option "--branch <name>", String, "The branch to create"
    c.option "--title <name>", String, "The title of the pull request"
    c.option "--body <text>", String, "The body of the pull request"
    c.option "--files <files,files,..>", String, "The files to check"
    c.example "Update Semaphore Ubuntu version",
              "renuo github-replace ubuntu1804 ubuntu2204 --projects=renuo-cli,renuo-website-v3 " \
              '--branch=feature/1234-update-ubuntu --title="Update os_image version" --body=TICKET-1234 ' \
              "--files=.semaphore/semaphore.yml"
    c.action { |args, options| new.run(args, options) }
  end

  def run(args, opts)
    validate_arguments args
    validate_options opts

    opts.projects.split(",").each do |repo|
      base = clone_and_checkout opts, repo
      find_and_replace args, repo, opts.files.split(",")
      create_pr opts, repo, base
    end
  end

  private

  def validate_arguments(args)
    abort ">> specify a search term and replace term" if args.size != 2
  end

  def validate_options(opts)
    abort ">> specify one or multiple comma-separated projects" unless opts.projects
    abort ">> specify a branch" unless opts.branch
    abort ">> specify a title" unless opts.title
    abort ">> specify a body" unless opts.body
    abort ">> specify comma-separated files to check" unless opts.files
  end

  def clone_and_checkout(opts, repo)
    puts `cd #{repo} || git clone --depth=1 git@github.com:renuo/#{repo}.git`
    puts `cd #{repo} && git checkout -b "#{opts.branch}"`
    `cd #{repo} && git branch -rl origin/main origin/master`.strip
  end

  def find_and_replace(args, repo, files)
    files.each do |file|
      file = "#{repo}/#{file}"
      puts "Replacing '#{args[0]}' with '#{args[1]}' in #{file}"
      content = File.read(file).gsub args[0], args[1]
      File.write file, content
    end
  end

  def create_pr(opts, repo, base)
    puts `cd "#{repo}" \
      && git add . \
      && git commit -S -m "#{opts.title}" \
      && git push --set-upstream origin "#{opts.branch}" \
      && gh pr create -a @me -b "#{opts.body}" -t "#{opts.title}" -B "#{base}" -H "#{opts.branch}"`
  end
end
